/***********************************************************
CookieManager definition
***********************************************************/
function CookieInfo(){
}
CookieInfo.prototype={
  getDomain:function(){
    if(this.domain){
      if(this.domain.charAt(0)!=".")return "."+this.domain;
      else return this.domain;
    }
    return this.uri.host;
  },
  getPath:function(){
    if(this.path)return this.path;
    var path=this.uri.path;
    var i=path.lastIndexOf("/");
    if(i==-1)return path;
    else return path.substring(0,i+1);
  },
  getExpiry:function(){
    if(!this.expires)return 0x7ffffffffffffdff;
    var exp=this.expires;
    exp=exp.replace(/-(.+?)-(\d{2}) /," $1 20$2 ").replace(/-/g," ");
    return Date.parse(exp)/1000;
  },
  toString:function(){
      var s=this.name+"="+this.value;
      for(var p in this){
        if(typeof this[p] == "function")continue;
        if(p=="httponly"||p=="secure"){
          s+="; "+p;
        }else if(p!="name"&&p!="value"&&p!="uri"){
          s+="; "+p+"="+this[p];
        }
      }
    return s;
  }
}
function CookieManager(){
  this.cookies=[];
  if("@mozilla.org/xre/app-info;1" in Components.classes){
    var appInfo = Components.classes["@mozilla.org/xre/app-info;1"]
                          .getService(Components.interfaces.nsIXULAppInfo);
    var versionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"]
                                 .getService(Components.interfaces.nsIVersionComparator);
    this.isNew=(versionChecker.compare(appInfo.platformVersion, "1.9") >= 0);
  }else{
    this.isNew=false;
  }
}
CookieManager.prototype={
  addCookies: function(aURI,aCookie){
    if(typeof(aURI)=="string"){
      var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                      .getService(Components.interfaces.nsIIOService);
      aURI=ioService.newURI(aURI,null,null);
    }
    aCookie=aCookie.split("\n");
    for each(var o in aCookie){
      var val=new CookieInfo();
      var ar=o.split(";");
      var t=ar[0].match(/(\S+?)=(.*)/);

      val["name"]=t[1];
      val["value"]=t[2];

      for(var i=1;i<ar.length;i++){
        if(ar[i]){
          var t=ar[i].replace( /^\s+/g,"");
          var f=t.match(/(\S+?)=(.*)/);
          if(f)val[f[1].toLowerCase()]=f[2];
          else{
            t=t.toLowerCase();
            if(t=="httponly"||t=="secure")val[t]=true;
            else val[t]="";
          }
        }
      }
      val.uri=aURI;
      this._insert(val);
//if(val.getPath()!=val.path)dout(val.getDomain()+" "+val.getPath()+" "+val.domain+" "+val.path);
    }
  },
  _insert: function(val){
    for(var i in this.cookies){
      var o=this.cookies[i];
      if(o.name==val.name&&o.getDomain()==val.getDomain()&&o.getPath()==val.getPath()){
        this.cookies[i]=val;
        return;
      }
    }
    this.cookies.push(val);
  },
  findCookie:function(urispec,name){
    for each(var o in this.cookies){
      if(o.uri.spec==urispec&&o.name==name){
        return o.toString();
      }
    }
    return null;
  },
  getCookie: function(aURI){
    var str="";
    for each(var o in this.cookies){
      var domain=o.getDomain();
      if(domain.charAt(0)==".")domain=domain.substring(1);
      if(aURI.host.lastIndexOf(domain)==-1)continue;
      if(aURI.path.indexOf(o.getPath())!=0)continue;
      if(o.secure&&aURI.scheme!="https")continue;
      var exp=o.expires;
      if(exp){
        exp=exp.replace(/-(.+?)-(\d{2}) /," $1 20$2 ").replace(/-/g," ");
        if(Date.parse(exp)<new Date()){
          continue;
        }
      }
      var s=o.name+"="+o.value;
      if(str)str+="; "+s;
      else str=s;
    }
    return str;
  },
  setCookieToBrowser: function(){
    var cm = Components.classes["@mozilla.org/cookiemanager;1"]
              .getService(Components.interfaces.nsICookieManager2);
    for each(var o in this.cookies){
//if(!o.expires)dout(o.name+" "+o.getExpiry());
      if (this.isNew){
        cm.add(o.getDomain(),o.getPath(),o.name,o.value,o.secure,o.httponly,!o.expires,o.getExpiry());
      }else{
        cm.add(o.getDomain(),o.getPath(),o.name,o.value,o.secure,!o.expires,o.getExpiry());
      }
    }
  },
  clear: function(){
    this.cookies=[];
  }
}
